import 'package:flutter/widgets.dart';
import 'package:forma/element_form.dart';
import 'i_form.dart';
import 'layouts/i_layout.dart';

class ElementForm extends StatefulWidget {
  // Final variables
  final Widget Function(BuildContext context, dynamic widgets) _build;
  final void Function(dynamic data) _onSave;
  final dynamic _elements;

  // Constructors
  ElementForm(dynamic elements,
      Widget Function(BuildContext context, dynamic widgets) build,
      {Key key, void Function(dynamic data) onSave})
      : _elements = elements,
        _build = build,
        _onSave = onSave,
        super(key: key);

  @override
  State<StatefulWidget> createState() =>
      ElementFormState<dynamic, ElementForm>();

  // Getters
  dynamic get elements => _elements;

  // Static factory methods
  static ElementForm withLayout(ILayout layout, dynamic elements,
      {Key key, void Function(dynamic data) onSave}) {
    return ElementForm(
      elements,
      layout.layout,
      key: key,
      onSave: onSave,
    );
  }
}

class ElementFormState<T, F extends ElementForm> extends IFormState<T, F> {
  GlobalKey<FormState> _formKey = GlobalKey<FormState>();

  @override
  void initState() {
    super.initState();
    // Initialize elements without modifying
    _mapElements<FormElement>((FormElement element) {
      element.initialize();
      return element;
    });
  }

  @override
  Widget build(BuildContext context) {
    // Build elements
    dynamic widgets = _mapElements<FormElement>((FormElement element) {
      return element.rebuild(context);
    });

    return Form(
      key: _formKey,
      child: widget._build(context, widgets),
    );
  }

  @override
  void dispose() {
    // Dispose all elements
    _mapElements<FormElement>((FormElement element) {
      element.dispose();
      return element;
    });

    super.dispose();
  }

  // Public methods
  @override
  T save() {
    if (_formKey.currentState.validate()) {
      // call on save events in case this is wanted
      _formKey.currentState.save();
      dynamic data = _mapElements<FormElement>((FormElement element) {
        return element.value;
      });

      if (widget._onSave != null) {
        widget._onSave(data);
      }
      return data;
    }

    return null;
  }

  // Private methods
  dynamic _mapElements<T>(dynamic Function(T) closure) {
    TreeMapper mapper = TreeMapper((dynamic node) {
      if (!(node is T)) {
        return null;
      }
      return closure(node as T);
    });

    return mapper.map(widget.elements);
  }
}
