import 'package:flutter/material.dart';
import 'package:forma/element_form.dart';
import 'package:forma/src/element_form/validators/i_validator.dart';

class MaterialTextFormElement extends FormElement {
  final Key _key;
  final String _initialValue;
  final InputDecoration _decoration;
  final TextStyle _style;
  final List<IValidator> _validators;

  TextEditingController _controller;

  @override
  get value => _controller.text;
  set value(dynamic value) => _controller.text = value;

  MaterialTextFormElement(
      {Key key,
      String initialValue,
      InputDecoration decoration = const InputDecoration(),
      TextStyle style,
      List<IValidator> validators})
      : _initialValue = initialValue,
        _decoration = decoration,
        _style = style,
        _validators = validators,
        _key = key;

  @override
  Widget build(BuildContext context) {
    return TextFormField(
      controller: _controller,
      key: _key,
      decoration: _decoration,
      style: _style,
      validator: _validate,
    );
  }

  @override
  void dispose() {
    _controller.dispose();
  }

  @override
  void initialize() {
    _controller = TextEditingController(text: _initialValue);
  }

  String _validate(String value) {
    if (_validators == null) {
      return null;
    }

    for (IValidator validator in _validators) {
      String errorText = validator.validate(value);
      if (errorText != null) {
        return errorText;
      }
    }

    return null;
  }
}
