import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:forma/element_form.dart';

void main() {
  group("Element form tests:", () {
    testWidgets('One element form test', (WidgetTester tester) async {
      Key textFieldKey = Key("textFieldKey");

      ElementForm form = ElementForm(TestElement(key: textFieldKey),
          (BuildContext context, dynamic widgets) {
        return widgets;
      });

      await tester.pumpWidget(MaterialApp(
        home: Scaffold(
          body: Center(
            child: form,
          ),
        ),
      ));

      expect(find.byKey(textFieldKey), findsOneWidget);
    });

    testWidgets("Return data test on form save", (WidgetTester tester) async {
      GlobalKey<ElementFormState> formKey = GlobalKey<ElementFormState>();

      ElementForm form = ElementForm({
        "field1": TestElement(initialValue: "Dataa"),
        "field2": TestElement(initialValue: "Field2Data")
      }, (BuildContext context, dynamic widgets) {
        return Column(
          children: <Widget>[
            SizedBox(
              child: widgets["field1"],
              height: 100,
              width: 200,
            ),
            SizedBox(
              child: widgets["field2"],
              height: 100,
              width: 200,
            )
          ],
        );
      }, key: formKey);

      await tester.pumpWidget(MaterialApp(
        home: Scaffold(
          body: Center(
            child: form,
          ),
        ),
      ));

      dynamic data = formKey.currentState.save();

      expect(data["field1"], "Dataa");
      expect(data["field2"], "Field2Data");
    });

    testWidgets("Auto layout widget array", (WidgetTester tester) async {
      GlobalKey<ElementFormState> formKey = GlobalKey<ElementFormState>();

      ElementForm form = ElementForm([
        TestElement(initialValue: "Dataa1"),
        TestElement(initialValue: "Dataa2")
      ], (BuildContext context, dynamic widgets) {
        return Column(children: widgets.cast<Widget>().toList());
      }, key: formKey);

      await tester.pumpWidget(MaterialApp(
        home: Scaffold(
          body: Center(
            child: form,
          ),
        ),
      ));

      dynamic data = formKey.currentState.save();

      expect(data[0], "Dataa1");
      expect(data[1], "Dataa2");
    });
  });
}

class TestElement extends FormElement {
  final Key key;
  final String _initialValue;
  TextEditingController _controller;

  @override
  get value => _controller.text;
  set value(dynamic value) => _controller.text = value;

  TestElement({this.key, String initialValue}) : _initialValue = initialValue;

  @override
  Widget build(BuildContext context) {
    return TextFormField(
      key: key,
      controller: _controller,
    );
  }

  @override
  dispose() {
    _controller.dispose();
  }

  @override
  initialize() {
    _controller = TextEditingController(text: _initialValue);
  }
}
