import 'package:flutter_test/flutter_test.dart';
import 'package:forma/element_form.dart';

void main() {
  group("Tree mapper tests:", () {
    test("Test single primitive int conversion", () {
      TreeMapper mapper = stringToIntMapper;
      expect(mapper.map("10"), 10);
    });

    test("Test array with primitive int conversion", () {
      TreeMapper mapper = stringToIntMapper;
      List<dynamic> result = mapper.map(["10", "20", "30"]);
      expect(result.last, 30);
    });

    test("Test hash with primitive int conversion", () {
      TreeMapper mapper = stringToIntMapper;
      Map<dynamic, dynamic> result = mapper.map({"int1": "40", "int2": "20"});
      expect(result["int1"], 40);
    });

    test("Test hash with nested array int conversion", () {
      TreeMapper mapper = stringToIntMapper;
      Map<dynamic, dynamic> result = mapper.map({
        "int1": "40",
        "nested": ["30", "50", "70"]
      });
      expect(result["nested"].last, 70);
    });

    test("Test array with child hashes", () {
      TreeMapper mapper = stringToIntMapper;
      List<dynamic> result = mapper.map([
        {"int1": "55"},
        {"int4": "155"}
      ]);
      expect(result.last["int4"], 155);
    });

    test("Test nested array int conversion", () {
      TreeMapper mapper = stringToIntMapper;
      List<dynamic> result = mapper.map([
        ["33", "88"],
        ["23"]
      ]);
      expect(result.first.last, 88);
    });

    test("Test conversion to unsupported type", () {
      TreeMapper mapper = stringToIntMapper;
      expect(() => mapper.map(10), throwsException);
    });

    test("Equality conversion test", () {
      TreeMapper mapper = equalityMapper;
      expect(mapper.map(10), 10);
    });

    test("Test multiple conversions", () {
      TreeMapper mapper = stringToIntMapper;
      Map<dynamic, dynamic> result = mapper.map({"int1": "40", "int2": "20"});
      Map<dynamic, dynamic> result2 = mapper.map({"int1": "40", "int2": "20"});

      expect(result["int1"], 40);
      expect(result2["int1"], 40);
    });
  });
}

TreeMapper get stringToIntMapper => TreeMapper((dynamic node) {
      if (!(node is String)) {
        return null;
      }

      return int.tryParse(node);
    });

TreeMapper get equalityMapper => TreeMapper((dynamic node) {
      return node;
    });
